import torch
from torch_geometric.nn import GeneralConv


def test_general_conv():
    x1 = torch.randn(4, 8)
    e1 = torch.randn(4, 16)
    edge_index = torch.tensor([[0, 1, 2, 3], [0, 0, 1, 1]])

    conv = GeneralConv(8, 32, 16)
    assert conv.__repr__() == 'GeneralConv(8, 32)'
    out = conv(x1, edge_index, edge_feature=e1)
    assert out.size() == (4, 32)
    assert conv(x1, edge_index, size=(4, 4),
                edge_feature=e1).tolist() == out.tolist()

    conv = GeneralConv(8, 32, 16, skip_linear=True)
    assert conv.__repr__() == 'GeneralConv(8, 32)'
    out = conv(x1, edge_index, edge_feature=e1)
    assert out.size() == (4, 32)
    assert conv(x1, edge_index, size=(4, 4),
                edge_feature=e1).tolist() == out.tolist()

    conv = GeneralConv(8, 32, 16, directed_msg=False)
    assert conv.__repr__() == 'GeneralConv(8, 32)'
    out = conv(x1, edge_index, edge_feature=e1)
    assert out.size() == (4, 32)
    assert conv(x1, edge_index, size=(4, 4),
                edge_feature=e1).tolist() == out.tolist()

    conv = GeneralConv(8, 32, 16, heads=3)
    assert conv.__repr__() == 'GeneralConv(8, 32)'
    out = conv(x1, edge_index, edge_feature=e1)
    assert out.size() == (4, 32)
    assert conv(x1, edge_index, size=(4, 4),
                edge_feature=e1).tolist() == out.tolist()

    conv = GeneralConv(8, 32, 16, attention=True)
    assert conv.__repr__() == 'GeneralConv(8, 32)'
    out = conv(x1, edge_index, edge_feature=e1)
    assert out.size() == (4, 32)
    assert conv(x1, edge_index, size=(4, 4),
                edge_feature=e1).tolist() == out.tolist()

    conv = GeneralConv(8, 32, 16, heads=3, attention=True)
    assert conv.__repr__() == 'GeneralConv(8, 32)'
    out = conv(x1, edge_index, edge_feature=e1)
    assert out.size() == (4, 32)
    assert conv(x1, edge_index, size=(4, 4),
                edge_feature=e1).tolist() == out.tolist()

    conv = GeneralConv(8, 32, 16, heads=3, attention=True,
                       attention_type='dot_product')
    assert conv.__repr__() == 'GeneralConv(8, 32)'
    out = conv(x1, edge_index, edge_feature=e1)
    assert out.size() == (4, 32)
    assert conv(x1, edge_index, size=(4, 4),
                edge_feature=e1).tolist() == out.tolist()

    conv = GeneralConv(8, 32, 16, l2_normalize=True)
    assert conv.__repr__() == 'GeneralConv(8, 32)'
    out = conv(x1, edge_index, edge_feature=e1)
    assert out.size() == (4, 32)
    assert conv(x1, edge_index, size=(4, 4),
                edge_feature=e1).tolist() == out.tolist()
